/*
 * Syslog_ServiceProcess.c
 *
 * Copyright (c) 2003, International Business Machines
 *
 * THIS FILE IS PROVIDED UNDER THE TERMS OF THE COMMON PUBLIC LICENSE 
 * ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION OF THIS FILE 
 * CONSTITUTES RECIPIENTS ACCEPTANCE OF THE AGREEMENT.
 *
 * You can obtain a current copy of the Common Public License from
 * http://oss.software.ibm.com/developerworks/opensource/license-cpl.html
 *
 * Author: R Sharada <sharada@in.ibm.com>
 *
 * Interface Type: Common Manageability Programming Interface ( CMPI )
 *
 * Description: CIM Linux Syslog Service Provider
 */


#include <cmpidt.h>
#include <cmpift.h>
#include <cmpimacs.h>
#include <string.h>
#include <libgen.h>
#include <cmpiOSBase_UnixProcess.h> // header for the cmpi-base common functions

#include "Syslog_ServiceUtils.h"
#include "util/syslogserviceutil.h"

#define LOCALCLASSNAME "Syslog_ServiceProcess"
#define SERVICECLASSNAME "Syslog_Service"
#define PROCESSCLASSNAME "Linux_UnixProcess"

extern int enum_all_process(struct processlist **lptr);

static const CMPIBroker *_broker;

#ifdef CMPI_VER_100
#define Syslog_ServiceProcessSetInstance Syslog_ServiceProcessModifyInstance
#endif

static CMPIStatus returnAssocName( const CMPIContext *ctx,
                                  const CMPIResult *rslt,
                                  const CMPIObjectPath *cop,
                                  char *clsname,
                                  Syslog_Service *buf)
{
  CMPIStatus st = {CMPI_RC_OK,NULL};
  CMPIObjectPath *op = NULL;
  Syslog_Service *servicebufp = NULL;
  struct processlist *processbufp = NULL;
  int cmdrc = 0;
  struct processlist *lptr = NULL;                                              
  struct cim_process *ptr = NULL;
  int pid, pid1, pid2, count = 0;

  syslog_debug(stderr, "--- returnAssocName()\n"); 

  if (strcasecmp(SERVICECLASSNAME,clsname)==0 && buf) {
    // build service object path from buffer 
    servicebufp = (Syslog_Service *)buf;
    op = makeServicePath(_broker,
                         clsname,
                         CMGetCharPtr(CMGetNameSpace(cop,NULL)),
                         servicebufp);
    if (CMIsNullObject(op)) {
      CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                           "Could not construct object path");
      return st;
    }
    CMReturnObjectPath(rslt,op);
  } // if clsname == SERVICECLASSNAME  

  else if (strcasecmp(PROCESSCLASSNAME,clsname)==0 && buf) {
    cmdrc = enum_all_process(&lptr);
    if (lptr) {
      processbufp = lptr; 
      servicebufp = (Syslog_Service *)buf;
      pid1 = servicebufp->syslogd;
      pid2 = servicebufp->klogd;
      if ((pid1 == 0) || (pid2 == 0)) { // the syslog service is not running 
	return st;	      
      }      
      while (processbufp->p != NULL) {
        ptr = processbufp->p;
        pid = atoi(ptr->pid);
        if (pid == pid1 || pid == pid2) {
          // build unixprocess object path from buffer 
          op = _makePath_UnixProcess(_broker, ctx, cop, ptr, &st);
          count++;
          if (CMIsNullObject(op)) {
            CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                                 "Could not construct object path");
            return st;
          }
          CMReturnObjectPath(rslt,op);
          if (count == 2) { 
            //return st;
            break;
          }
        } // if matching pid  
        processbufp = processbufp->next;
      } // end while 
    } // if lptr 
  } // if clsname == PROCESSCLASSNAME 

  else {
    char mesg[1024];
    sprintf(mesg, "--- unsupported class %s\n",CMGetCharPtr(CMGetClassName(cop,NULL)));
    syslog_debug(stderr, mesg);
  } // else unsupported class 
  
  return st;
}

static CMPIStatus returnAssoc(const CMPIContext *ctx,
                              const CMPIResult *rslt,
                              const CMPIObjectPath *cop,
                              char *clsname,
                              Syslog_Service *buf )
{
  CMPIStatus st = {CMPI_RC_OK,NULL};
  CMPIInstance *in = NULL;
  Syslog_Service *servicebufp = NULL;
  struct processlist *processbufp = NULL;
  int cmdrc = 0;
  struct processlist *lptr = NULL;
  struct cim_process *ptr = NULL;
  int pid, pid1, pid2, count = 0;
  const char **properties;

  syslog_debug(stderr, "--- returnAssoc()\n");

  if (strcasecmp(SERVICECLASSNAME,clsname)==0 && buf) {
    // build service object instance from buffer 
    servicebufp = (Syslog_Service *)buf;
    in = makeServiceInstance(_broker, 
		             clsname, 
			     CMGetCharPtr(CMGetNameSpace(cop,NULL)),
			     servicebufp );
    if (CMIsNullObject(in)) {
      CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                           "Could not construct object instance");
      return st;
    }
    CMReturnInstance(rslt,in);
  } // if clsname == SERVICECLASSNAME 

  else if (strcasecmp(PROCESSCLASSNAME,clsname)==0 && buf) {
    cmdrc = enum_all_process(&lptr);
    if (lptr) {
      processbufp = lptr;
      servicebufp = (Syslog_Service *)buf;
      pid1 = servicebufp->syslogd;
      pid2 = servicebufp->klogd;
      if ((pid1 == 0) || (pid2 == 0)) { // the syslog service is not running
        return st;
      }
      while (processbufp->p != NULL) {
        ptr = processbufp->p;
        pid = atoi(ptr->pid);
        if (pid == pid1 || pid == pid2) {
          // build unixprocess object instance from buffer 
          in = _makeInst_UnixProcess(_broker, ctx, cop, properties, ptr, &st);
          count++;
          if (CMIsNullObject(in)) {
            CMSetStatusWithChars( _broker, &st, CMPI_RC_ERR_FAILED,
                                  "Could not construct object path" );
            return st;
          }
	  CMReturnInstance(rslt,in);
          if (count == 2) {
	    return st;
	  }
        } // if matching pid 
        processbufp = processbufp->next;
      } // end while 
    } // if lptr 
  } // if clsname == PROCESSCLASSNAME 
                                                                                
  else {
    char mesg[1024];
    sprintf(mesg, "--- unsupported class %s\n",CMGetCharPtr(CMGetClassName(cop,NULL)));
    syslog_debug(stderr, mesg);
  } // else unsupported class 
                                                                                
  return st;
}

static CMPIStatus returnRefName( const CMPIContext *ctx,
                                const CMPIResult *rslt,
                                const CMPIObjectPath *cop,
                                char *clsname,
                                Syslog_Service *buf)
{
  CMPIStatus st = {CMPI_RC_OK,NULL};
  CMPIObjectPath *op = NULL;
  CMPIObjectPath *opRef = NULL;
  CMPIData dt;
  Syslog_Service *servicebufp = NULL;
  struct processlist *processbufp = NULL;
  int cmdrc = 0;
  struct processlist *lptr = NULL;
  struct cim_process *ptr = NULL;
  int pid, pid1, pid2, count = 0;
  char *procpid;
                                                                                
  syslog_debug(stderr, "--- returnRefName()\n");

  if (strcasecmp(PROCESSCLASSNAME,clsname)==0) {
    cmdrc = enum_all_process(&lptr);
    if (lptr) {
      processbufp = lptr;
      servicebufp = (Syslog_Service *)buf;
      pid1 = servicebufp->syslogd;
      pid2 = servicebufp->klogd;
      if ((pid1 == 0) || (pid2 == 0)) { // the syslog service is not active
	return st;      
      }	      
      while (processbufp->p != NULL) {
        ptr = processbufp->p;
        pid = atoi(ptr->pid);
        if (pid == pid1 || pid == pid2)  { 
          // build unixprocess object path from buffer
          op = _makePath_UnixProcess(_broker, ctx, cop, ptr, &st);
          count ++;
          if (CMIsNullObject(op)) {
            CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                                 "Could not construct object path");
            return st;
          }
          // make reference object path 
          opRef = CMNewObjectPath(_broker,
                                  CMGetCharPtr(CMGetNameSpace(cop,&st)),
                                  LOCALCLASSNAME,
                                  &st);
          if (CMIsNullObject(opRef)) {
            CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                                 "Could not construct object path");
            return st;
          }
          CMAddKey(opRef,"process",&op,CMPI_ref);
          CMAddKey(opRef,"service",&cop,CMPI_ref);
          CMReturnObjectPath(rslt,opRef);
          if ( count == 2) {
            break;
          }
        } // if pid matching 
        processbufp = processbufp->next;
      } // end while 
    } // if lptr 	      
  } // if clsname == PROCESSCLASSNAME 

  else if (strcasecmp(clsname, SERVICECLASSNAME)==0) {
    dt = CMGetKey(cop, "Handle", &st);
    procpid = CMGetCharPtr(dt.value.string);
    pid = atoi(procpid);
    if (st.rc != CMPI_RC_OK) {
      CMSetStatusWithChars(_broker,&st,CMPI_RC_ERR_FAILED,
                           "Could not get handle");
    }  
    servicebufp = (Syslog_Service *)buf;
    // build service object path from buffer
    op = makeServicePath(_broker,
                         clsname,
                         CMGetCharPtr(CMGetNameSpace(cop,NULL)),
                         servicebufp);
    if (CMIsNullObject(op)) {
      CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                           "Could not construct object path");
      return st;
    }
    pid1 = servicebufp->syslogd;
    pid2 = servicebufp->klogd;
    if ((pid1 == 0) || (pid2 == 0)) { // the syslog service is not active
	return st;      
    }	      
    if ((pid == pid1) || (pid == pid2)) {
      // make reference object path 
      opRef = CMNewObjectPath(_broker,
                              CMGetCharPtr(CMGetNameSpace(cop,&st)),
                              LOCALCLASSNAME,
                              &st);
      if (CMIsNullObject(opRef)) {
        CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                             "Could not construct object path");
        return st;
      }
      CMAddKey(opRef,"process",&cop,CMPI_ref);
      CMAddKey(opRef,"service",&op,CMPI_ref);
      CMReturnObjectPath(rslt,opRef);
    }	    
  } // else if clsname == SERVICECLASSNAME 

  else {
   char mesg[1024];
   sprintf(mesg, "--- unsupported class %s\n",CMGetCharPtr(CMGetClassName(cop,NULL)));
   syslog_debug(stderr, mesg);
  } // else unsupported class 

  return st;
}

static CMPIStatus returnRef( const CMPIContext *ctx,
                            const CMPIResult *rslt,
                            const CMPIObjectPath *cop,
                            char *clsname,
                            Syslog_Service *buf )
{
  CMPIStatus st = {CMPI_RC_OK,NULL};
  CMPIObjectPath *op = NULL;
  CMPIObjectPath *opRef = NULL;
  CMPIInstance *inRef = NULL;
  CMPIData dt;
  Syslog_Service *servicebufp = NULL;
  struct processlist *processbufp = NULL;
  int cmdrc = 0;
  struct processlist *lptr = NULL;
  struct cim_process *ptr = NULL;
  int pid, pid1, pid2, count = 0;
  char *procpid;

  syslog_debug(stderr, "--- returnRef()\n");

  if (strcasecmp(PROCESSCLASSNAME,clsname)==0) {
    cmdrc = enum_all_process(&lptr);
    if (lptr) {
      processbufp = lptr;
      servicebufp = (Syslog_Service *)buf;
      pid1 = servicebufp->syslogd;
      pid2 = servicebufp->klogd;
      if ((pid1 == 0) || (pid2 == 0)) { // the syslog service is not active
        return st;
      }
      while (processbufp->p != NULL) {
        ptr = processbufp->p;
        pid = atoi(ptr->pid);
        if (pid == pid1 || pid == pid2)  {
          // build unixprocess object path from buffer
          op = _makePath_UnixProcess(_broker, ctx, cop, ptr, &st );
          count ++;
	  if (CMIsNullObject(op)) {
            CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                                 "Could not construct object path" );
            return st;
          }
	  // make reference object path 
	  opRef = CMNewObjectPath(_broker,
				  CMGetCharPtr(CMGetNameSpace(cop,&st)),
				  LOCALCLASSNAME,
				  &st );
          if (CMIsNullObject(opRef)) {
            CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			         "Could not construct object path" );
            return st;
          }
	  // make reference object instance
	  inRef = CMNewInstance(_broker,opRef,NULL);
          if (CMIsNullObject(inRef)) {
            CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
				 "Could not construct object instance" );
            return st;
          }

	  CMSetProperty(inRef,"process",&op,CMPI_ref);
	  CMSetProperty(inRef,"service",&cop,CMPI_ref);
	  CMReturnInstance(rslt,inRef);

          if (count == 2) {
            break;
          }
        } // if pid matching 
        processbufp = processbufp->next;
      } // end while 
    } // if lptr 
  } // if clsname == PROCESSCLASSNAME 

  else if (strcasecmp(clsname, SERVICECLASSNAME)==0) {
    dt = CMGetKey(cop, "Handle", &st);
    procpid = CMGetCharPtr(dt.value.string);
    pid = atoi(procpid);
    if (st.rc != CMPI_RC_OK) {
      CMSetStatusWithChars(_broker,&st,CMPI_RC_ERR_FAILED,
			   "Could not get handle" );
    }
    servicebufp=(Syslog_Service *)buf;
    // build service object path from buffer
    op = makeServicePath(_broker,
			 clsname,
			 CMGetCharPtr(CMGetNameSpace(cop,NULL)),
			 servicebufp );
    if (CMIsNullObject(op)) {
      CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
			   "Could not construct object path" );
      return st;
    }
    pid1 = servicebufp->syslogd;
    pid2 = servicebufp->klogd;
    if ((pid1 == 0) || (pid2 == 0)) { // the syslog service is not active
      return st;
    }
    if ((pid == pid1) || (pid == pid2)) {
      // make reference object path 
      opRef = CMNewObjectPath(_broker,
                              CMGetCharPtr(CMGetNameSpace(cop,&st)),
			      LOCALCLASSNAME,
			      &st );
      if (CMIsNullObject(opRef)) {
        CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                             "Could not construct object path");
        return st;
      }
      // make reference object instance
      inRef = CMNewInstance(_broker,opRef,NULL);
      if (CMIsNullObject(inRef)) {
        CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                             "Could not construct object instance");
        return st;
      }

      CMSetProperty(inRef,"process",&cop,CMPI_ref);
      CMSetProperty(inRef,"service",&op,CMPI_ref);
      CMReturnInstance(rslt,inRef);
    }
  } // else if clsname == SERVICECLASSNAME 

  else {
    char mesg[1024];
    sprintf(mesg, "--- unsupported class %s\n",CMGetCharPtr(CMGetClassName(cop,NULL)));
    syslog_debug(stderr, mesg);
  } // else unsupported class 

  return st;
}

static CMPIStatus associatorHelper( const CMPIContext *ctx,
                                   const  CMPIResult *rslt,
                                   const CMPIObjectPath *cop,
                                   int associators, 
				   int names )
{
  CMPIStatus st = {CMPI_RC_OK,NULL};
  CMPIString *clsname;
  CMPIData data;
  void *enumhdl;
  Syslog_Service servicebuf;

  syslog_debug(stderr,"--- associatorHelper()\n");

  //  Check if the object path belongs to a supported class
  clsname = CMGetClassName(cop,NULL);
  if (clsname) {
    if (strcasecmp(PROCESSCLASSNAME,CMGetCharPtr(clsname))==0) {
      // we have the Process and can return the Service 
      data = CMGetKey(cop,"Name",NULL);
      enumhdl = Syslog_Service_Begin_Enum();
      if (enumhdl == NULL) {
        CMSetStatusWithChars(_broker, &st, CMPI_RC_ERR_FAILED,
                             "Could not begin syslog service enumeration");
        return st;
      } 
      else {
        if (Syslog_Service_Next_Enum(enumhdl,&servicebuf)) {
          if (names) {
            if (associators) {
              st = returnAssocName(ctx, rslt, cop, SERVICECLASSNAME, &servicebuf);
            } 
	    else {
              st = returnRefName(ctx, rslt, cop, SERVICECLASSNAME, &servicebuf);
            }
          } // if names, return paths
	  else {
	    if (associators) {
	      st = returnAssoc(ctx, rslt, cop, SERVICECLASSNAME, &servicebuf);
	    }
            else {
              st = returnRef(ctx, rslt, cop, SERVICECLASSNAME, &servicebuf);
            }
          } // if !names, return instances
        } // syslog_service_next_enum()
        Syslog_Service_End_Enum(enumhdl);
      }
    } // if clsname == PROCESSCLASSNAME 

    else if (strcasecmp(SERVICECLASSNAME,CMGetCharPtr(clsname))==0) {
      // we have the Service and can return the Processes
      data = CMGetKey(cop,"Name",NULL);
      enumhdl = Syslog_Service_Begin_Enum();
      if (Syslog_Service_Next_Enum(enumhdl,&servicebuf)) {
        if (names) {
          if (associators) {
            st = returnAssocName(ctx, rslt, cop, PROCESSCLASSNAME, &servicebuf);
          } 
	  else {
            st = returnRefName(ctx, rslt, cop, PROCESSCLASSNAME, &servicebuf);
          }
        } // if names 
	else {
          if (associators) {
            st = returnAssoc(ctx, rslt, cop, PROCESSCLASSNAME, &servicebuf);
          }
          else {
            st = returnRef(ctx, rslt, cop, PROCESSCLASSNAME, &servicebuf);
          }
        } // if !names, return instances
      } // syslog_service_next_enum()
      Syslog_Service_End_Enum(enumhdl);
    } // else if clsname == SERVICECLASSNAME 
                                                                                
    else {
      char mesg[1024];
      sprintf(mesg, "--- unsupported class %s\n",CMGetCharPtr(clsname));
      syslog_debug(stderr, mesg);
    } // else unsupported class

    CMReturnDone(rslt);
  } // if (clsname) 

  return st;
}

/* Instance MI Cleanup */

CMPIStatus Syslog_ServiceProcessCleanup( CMPIInstanceMI *mi, 
                                        const CMPIContext *ctx,
					CMPIBoolean terminate) 
{
  CMReturn(CMPI_RC_OK);
}

/* Instance MI Functions */

CMPIStatus Syslog_ServiceProcessEnumInstanceNames( CMPIInstanceMI *mi, 
                                                  const CMPIContext *ctx, 
                                                  const CMPIResult *rslt, 
                                                  const CMPIObjectPath *ref) 
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessEnumInstanceNames()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );	
}

CMPIStatus Syslog_ServiceProcessEnumInstances( CMPIInstanceMI *mi, 
                                              const CMPIContext *ctx, 
                                              const CMPIResult *rslt, 
                                              const CMPIObjectPath *ref, 
                                              const char **properties) 
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessEnumInstances()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );	
}

CMPIStatus Syslog_ServiceProcessGetInstance( CMPIInstanceMI *mi, 
                                            const CMPIContext *ctx, 
                                            const CMPIResult *rslt, 
                                            const CMPIObjectPath *cop, 
                                            const char **properties) 
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessGetInstance()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );	
}

CMPIStatus Syslog_ServiceProcessCreateInstance( CMPIInstanceMI *mi, 
                                               const CMPIContext *ctx, 
                                               const CMPIResult *rslt, 
                                               const CMPIObjectPath *cop, 
                                               const CMPIInstance *ci) 
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessCreateInstance()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus Syslog_ServiceProcessSetInstance( CMPIInstanceMI *mi, 
                                            const CMPIContext *ctx, 
                                            const CMPIResult *rslt, 
                                            const CMPIObjectPath *cop,
                                            const CMPIInstance *ci, 
                                            const char **properties) 
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessSetInstance()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus Syslog_ServiceProcessDeleteInstance( CMPIInstanceMI *mi, 
                                               const CMPIContext *ctx, 
                                               const CMPIResult *rslt, 
                                               const CMPIObjectPath *cop) 
{ 
  syslog_debug(stderr, "--- Syslog_ServiceProcessDeleteInstance()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

CMPIStatus Syslog_ServiceProcessExecQuery( CMPIInstanceMI *mi, 
                                          const CMPIContext *ctx, 
                                          const CMPIResult *rslt, 
                                          const CMPIObjectPath *cop, 
                                          const char *lang, 
                                          const char *query) 
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessExecQuery()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
}

/* Association MI Cleanup */

CMPIStatus Syslog_ServiceProcessAssociationCleanup( CMPIAssociationMI *mi,
                                                    const CMPIContext *ctx,
						CMPIBoolean terminate)
{
  CMReturn(CMPI_RC_OK);
}

/* Association MI Functions */                                        
                                                                                
CMPIStatus Syslog_ServiceProcessAssociators( CMPIAssociationMI *mi,
                                            const CMPIContext *ctx,
                                            const  CMPIResult *rslt,
                                            const CMPIObjectPath *cop,
                                            const char *assocClass,
                                            const char *resultClass,
                                            const char *role,
                                            const char *resultRole,
                                            const char **propertyList)
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessAssociators()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
  return associatorHelper(ctx, rslt, cop, 1, 0);
}

CMPIStatus Syslog_ServiceProcessAssociatorNames( CMPIAssociationMI *mi,
                                                const CMPIContext *ctx,
                                                const CMPIResult *rslt,
                                                const CMPIObjectPath *cop,
                                                const char *assocClass,
                                                const char *resultClass,
                                                const char *role,
                                                const char *resultRole )
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessAssociatorNames()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
  return associatorHelper(ctx, rslt, cop, 1, 1);
}

CMPIStatus Syslog_ServiceProcessReferences( CMPIAssociationMI *mi,
                                           const CMPIContext *ctx,
                                           const CMPIResult *rslt,
                                           const CMPIObjectPath *cop,
                                           const char *assocClass,
                                           const char *role,
                                           const char **propertyList )
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessReferences()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
  return associatorHelper(ctx, rslt, cop, 0, 0);
}

CMPIStatus Syslog_ServiceProcessReferenceNames( CMPIAssociationMI *mi,
                                               const CMPIContext *ctx,
                                               const CMPIResult *rslt,
                                               const CMPIObjectPath *cop,
                                               const char *assocClass,
                                               const char *role)
{
  syslog_debug(stderr, "--- Syslog_ServiceProcessReferenceNames()\n");
  CMReturn( CMPI_RC_ERR_NOT_SUPPORTED );
  return associatorHelper(ctx, rslt, cop, 0, 1);
}

/* Instance MI Factory */

CMInstanceMIStub( Syslog_ServiceProcess,
                  Syslog_ServiceProcess,
                  _broker,
                  CMNoHook);


/* Association MI Factory */                                          
                                                                                
CMAssociationMIStub( Syslog_ServiceProcess,
                     Syslog_ServiceProcess,
                     _broker,
                     CMNoHook);

